use crate::file_watcher::prelude::*;

use crate::Besult;
use axum::Router;
use std::path::PathBuf;
use std::str::FromStr;
use std::sync::OnceLock;
use tower_http::services::ServeDir;
use tower_livereload::Reloader;
use tracing::{info, instrument};

fn static_file_dir() -> &'static PathBuf {
    static STATIC_FILE_DIR: OnceLock<PathBuf> = OnceLock::new();
    STATIC_FILE_DIR.get_or_init(|| PathBuf::from_str("static").unwrap())
}

#[instrument]
pub fn router(reloader: Option<Reloader>) -> Besult<(Router, Option<FileWatcher>)> {
    let watcher = if let Some(rl) = reloader {
        Some(file_watcher(static_file_dir(), move |ev| {
            info!("Static File Watcher Event: {ev:#?}");
            rl.reload()
        })?)
    } else {
        None
    };

    let router = Router::new().nest_service("/", ServeDir::new(static_file_dir()));
    Ok((router, watcher))
}
